var FlowCell = function()
{
	var flowcell = {};
	var debug = 0;
	var idField;
	var flowCellsAreValid = false;

	var commentsIsValid = [];
	var flowCellIdIsValid = [];

	// Page initialization
	flowcell.initPage = function()
	{
		// Step 1
		Events.addEventHandler('flowcells', 'change', flowcell.flowCellsOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', flowcell.validateStep1);
		Events.addEventHandler('outcomeSuccess', 'click', flowcell.outcomeOnChange);
		Events.addEventHandler('outcomeFailed', 'click', flowcell.outcomeOnChange);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', flowcell.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', flowcell.validateStep2);
		Events.addEventHandler('clusterDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('fc-0.externalId', 'change', flowcell.flowCellIdOnChange);
		Events.addEventHandler('fc-1.externalId', 'change', flowcell.flowCellIdOnChange);
		Events.addEventHandler('fc-0.comments', 'change', flowcell.commentsOnChange);
		Events.addEventHandler('fc-1.comments', 'change', flowcell.commentsOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', flowcell.submit);

		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnprocessedFlowCells';		
		Wizard.showLoadingAnimation('Loading flow cells...');
		Wizard.asyncJsonRequest(url, flowcell.initializeStep1);
	}


	flowcell.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var flowCells = response.flowCells;
		
		if (flowCells.length > 0)
		{
			for (var fcNo=0; fcNo < flowCells.length; fcNo++)
			{
				var fc = flowCells[fcNo];
				var option = flowcell.getOptionForFlowCell(fc, fcNo);
				frm.flowcells[frm.flowcells.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No flow cells available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
		
		flowcell.flowCellsOnChange();
		flowcell.outcomeOnChange();
		frm.flowcells.focus();
	}
	
	flowcell.getOptionForFlowCell = function(fc, fcNo)
	{
		var name = fc.name +': ';

		var numPools = fc.pools.length;
		var firstPoolNum = flowcell.getPoolNum(fc.pools[0].name);
		var lastPoolNum = flowcell.getPoolNum(fc.pools[fc.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += fc.pools[0].name + ' — ' + fc.pools[fc.pools.length-1].name;
			}
			else
			{
				name += fc.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += fc.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}
		
		if (fc.FlowCellType)
		{
			name += ' [' + fc.FlowCellType + ']';
		}

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = fc.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}
		
		var option = new Option(name, fc.id, fcNo < 2);
		option.title = title;
		option.flowCell = fc;

		return option;
	}
	
	flowcell.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;	
	}

	flowcell.flowCellsOnChange = function()
	{
		var frm = document.forms['reggie'];	
		flowCellsAreValid = false;
		
		var numSelected = 0;
		for (var i = 0; i < frm.flowcells.length; i++)
		{
			if (frm.flowcells[i].selected) numSelected++;
		}
		
		if (numSelected < 1 || numSelected > 2)
		{
			Wizard.setInputStatus('flowcells', 'invalid', 'Select 1 or 2 flow cells');
			return false;
		}
		
		Wizard.setInputStatus('flowcells', 'valid');
		flowCellsAreValid = true;
	}

	flowcell.outcomeOnChange = function()
	{
		var frm = document.forms['reggie'];
		var failed = Doc.element('outcomeFailed').checked;
		Doc.addOrRemoveClass(frm['fc-0.comments'], 'required', failed);
		Doc.addOrRemoveClass(frm['fc-1.comments'], 'required', failed);
	}

	
	flowcell.validateStep1 = function(event)
	{
		if (!flowCellsAreValid) event.preventDefault();
	}
	
	flowcell.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		var failed = frm.outcome[1].checked;
		if (failed)
		{
			Wizard.showGoNextConfirmation(true, 'Check to verify registration of failure');
		}

		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');
		frm.clusterDate.focus();
		
		var index = 0;
		var flowCellType = null;
		for (var i = 0; i < frm.flowcells.length && index < 2; i++)
		{
			if (frm.flowcells[i].selected)
			{
				var fc = frm.flowcells[i].flowCell;
				if (flowCellType == null) flowCellType = fc.FlowCellType;
				var prefix = 'fc-'+index;
				Doc.element(prefix+'.name.c').innerHTML = Strings.encodeTags(frm.flowcells[i].text);
				frm[prefix+'.comments'].value = fc.comments;
				Events.sendChangeEvent(prefix+'.comments');
				frm[prefix+'.externalId'].value = fc.externalId;
				Events.sendChangeEvent(prefix+'.externalId');
				index++;
			}
		}
		
		if (index == 1)
		{
			Doc.hide('fc-1-section');
		}
		
		// Load clustering protocols
		Reggie.loadProtocols('CLUSTER_PROTOCOL', 'clusterProtocol');
		Reggie.loadHardware(flowCellType == 'NextSeq' ? 'SEQUENCER' : 'CLUSTER_GENERATOR', 'cBot');
	}
	
	// Cluster comments are required when 'Failure' has been selected
	flowcell.commentsOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var commentField = event.currentTarget.id;
		
		commentsIsValid[commentField] = false;
		Wizard.setInputStatus(commentField);
		
		var comments = frm[commentField].value;
		if (comments == '')
		{
			var required = frm[commentField].className.indexOf('required') != -1;
			if (required)
			{
				Wizard.setInputStatus(commentField, 'invalid', 'Missing');
				return;
			}
		}

		Wizard.setInputStatus(commentField, 'valid');
		commentsIsValid[commentField] = true;
	}

	flowcell.flowCellIdOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		idField = event.currentTarget.id;
		
		Wizard.setInputStatus(idField);
		flowCellIdIsValid[idField] = false;
		
		var id = frm[idField].value;
		if (id == '')
		{
			Wizard.setInputStatus(idField, 'invalid', 'ID is a required value');
			return;
		}
		
		var otherIdField = idField == 'fc-1.externalId' ? 'fc-0.externalId' : 'fc-1.externalId';
		if (id == frm[otherIdField].value)
		{
			Wizard.setInputStatus(idField, 'invalid', 'ID must be different');
			return;
		}
		
		// Check if ID already exists in DB
		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlowCellByBarcode';
		url += '&barcode='+encodeURIComponent(id);
		Wizard.showLoadingAnimation('Checking flow cell id...');
		Wizard.asyncJsonRequest(url, flowcell.flowCellIdChecked);
	}
	
	flowcell.flowCellIdChecked = function(response)
	{
		var frm = document.forms['reggie'];
		var existing = response.flowCell;
		if (existing != null)
		{
			var id = frm[idField].value;
			Wizard.setInputStatus(idField, 'invalid', 'Flow cell with ID='+Strings.encodeTags(id)+' already exists: ' + Strings.encodeTags(existing.name));
			return;
		}
		Wizard.setInputStatus(idField, 'valid');
		flowCellIdIsValid[idField] = true;
	}

	flowcell.validateStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		var failed = frm.outcome[1].checked;
		
		if (!Wizard.isValid('clusterDate')) 
		{
			event.preventDefault();
			return;
		}
		
		var index = 0;
		for (var i = 0; i < frm.flowcells.length; i++)
		{
			if (frm.flowcells[i].selected)
			{
				var fc = frm.flowcells[i].flowCell;
				if (flowCellIdIsValid['fc-'+index+'.externalId'] != true) 
				{
					Wizard.setInputStatus('fc-'+index+'.externalId', 'invalid');
					event.preventDefault();
					return;
				}
				if (commentsIsValid['fc-'+index+'.comments'] != true)
				{
					Wizard.setInputStatus('fc-'+index+'.comments', 'invalid');
					event.preventDefault();
					return;
				}
				index++;
			}
		}
	}


	flowcell.submit = function()
	{
		var frm = document.forms['reggie'];
		var failed = Doc.element('outcomeFailed').checked;

		var submitInfo = {};
		submitInfo.failed = failed;
		submitInfo.flowCells = [];
		
		var index = 0;
		for (var i = 0; i < frm.flowcells.length; i++)
		{
			if (frm.flowcells[i].selected) 
			{
				var fc = frm.flowcells[i].flowCell;
				fc.externalId = frm['fc-'+index+'.externalId'].value;
				fc.clusterDate = frm.clusterDate.value;
				fc.clusterOperator = frm.clusterOperator.value;
				fc.cBot = parseInt(frm.cBot.value, 10);
				fc.clusterProtocol = parseInt(frm.clusterProtocol.value, 10);
				fc.comments = frm['fc-'+index+'.comments'].value;
				submitInfo.flowCells[submitInfo.flowCells.length] = fc;
				index++;
			}
		}
		
		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterFlowCells';
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, flowcell.submissionResults, 'POST', JSON.stringify(submitInfo));
		
	}
	
	flowcell.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return flowcell;
}();

Doc.onLoad(FlowCell.initPage);

